@@ -1,5 +1,5 @@
 /* cdrecord.c
- * Copyright (C) 2004, 2005 Sylvain Cresto <sylvain.cresto@tiscali.fr>
+ * Copyright (C) 2004, 2005 Sylvain Cresto <scresto@gmail.com>
  *
  * This file is part of graveman!
  *
@@ -38,6 +38,7 @@
 #define CDRECORD_INAPPROPRIATE "Inappropriate audio coding"
 #define CDRECORD_INPUTOUTPUT_ERROR "Input/output error."
 #define CDRECORD_FIXATING "Fixating..."
+#define CDRECORD_BLANKING "Blanking "
 #define CDRECORD_BAD_RECORDER "Sorry, no CD/DVD-Recorder or unsupported CD/DVD-Recorder found"
 
 /* support du materiel */
@@ -47,46 +48,50 @@
 #define CDRECORD_WRITE_CDR "Does write CD-R media"
 #define CDRECORD_NOT_WRITE_CDRW "Does not write CD-RW media"
 #define CDRECORD_WRITE_CDRW "Does write CD-RW media"
+#define CDRECORD_READ_DVD "Does read DVD-ROM media"
+#define CDRECORD_WRITE_DVD "Does write DVD-R media"
+#define CDRECORD_NOT_WRITE_DVD "Does not write DVD-R media"
 #define CDRECORD_NOT_WRITE_DUMMY "Does not support test writing"
 #define CDRECORD_WRITE_DUMMY "Does support test writing"
 #define CDRECORD_MAX_CDREAD_SPEED "Maximum read  speed:"
 #define CDRECORD_MAX_CDREAD_SPEED_CD "(CD"
+#define CDRECORD_MAX_CDREAD_SPEED_DVD " DVD "
 
 /* effacer un cdrw ou cloturer un cdr */
 #define CDRECORD_BLANKING_TIME "Blanking time:"
 #define CDRECORD_CANNOT_BLANK "Cannot blank disk, aborting"
-#define CDRECORD_INCOMPATIBLE_MEDIUM "cannot format medium - incompatible medium"
+#define CDRECORD_INCOMPATIBLE_MEDIUM "cannot format medium - incmedium"
 #define CDRECORD_FIXATING_TIME "Fixating time:"
 
 
 /* definition de tous les type de bus */
 Tsearchdrive listesearchdrives[] = {
-    { "", "", " ", 100 }, /* scsi (sg) */
-    { "dev=ATA:", "ATA:", "ATA", 80 }, /* ide sg */
-    { "dev=ATAPI:", "ATAPI:", "ATAPI", 60 }, /* ide */
+    { "", "", "SCSI", 100 }, /* scsi (sg) */
 #ifdef LINUX_IDE  /* pure ide devices with linux */
     { "dev=/dev/hda", "/dev/hda", "IDE", 40 },
     { "dev=/dev/hdb", "/dev/hdb", "IDE", 40 },
     { "dev=/dev/hdc", "/dev/hdc", "IDE", 40},
     { "dev=/dev/hdd", "/dev/hdd", "IDE", 40 },
 #endif
+    { "dev=ATA:", "ATA:", "ATA", 80 }, /* ide sg */
+    { "dev=ATAPI:", "ATAPI:", "ATAPI", 60 }, /* ide */
     { NULL, NULL, NULL, 0}
   };
 
 gboolean cdrecord_grave_callback(GIOChannel *Astd, GIOCondition Acond, gpointer Adata);
 
-gchar *get_title(gint Anbrgravure, gint Acurcd, gboolean Adosimul)
+static gchar *get_title(gint Anbrgravure, gint Acurcd, gboolean Adosimul)
 {
   gchar *Ltxt = NULL;
   if (Anbrgravure==1) {
     if (!Adosimul) {
       Ltxt = g_strdup(_("Writing in progress..."));
     } else {
-      Ltxt = g_strdup(_("AABB Simulated CD writing in progress..."));
+      Ltxt = g_strdup(_("Simulated CD writing in progress..."));
     }
   } else {
     if (!Adosimul) {
-      Ltxt = g_strdup_printf(_("Writing cd %d/%d in progress..."), Acurcd, Anbrgravure);
+      Ltxt = g_strdup_printf(_("Writing DVD %d/%d in progress..."), Acurcd, Anbrgravure);
     } else {
       Ltxt = g_strdup_printf(_("Simulated CD writing %d/%d in progress..."), Acurcd, Anbrgravure);
     }
@@ -145,7 +150,7 @@ gboolean scan_callback(GIOChannel *Astd, GIOCondition Acond, gpointer Adata)
   }
   
   Lstatus = g_io_channel_read_line(Astd, &Lbuffer, NULL, NULL, NULL);  
-  if (!Lbuffer) { *Lcont = FALSE; return FALSE; }
+  if (!Lbuffer) return FALSE;
     
   g_strstrip(Lbuffer);  
 /* _DEB("scan lecteur = %s\n", Lbuffer); */
@@ -178,7 +183,7 @@ gboolean scan_for_drives(GHashTable *Ahash, GError **Aerror)
 {
   gchar **Lcmd;
   gchar *Lcommandline;
-  gint pid, g_out, Lnbrarg;
+  gint Lpid, g_out, Lnbrarg;
   gboolean Lbolstatus;
   GIOChannel *Lcom;
   gboolean Lstatus;
@@ -201,7 +206,7 @@ gboolean scan_for_drives(GHashTable *Ahash, GError **Aerror)
 
     Lbolstatus = g_spawn_async_with_pipes(NULL, Lcmd, NULL, /* env argument */
       (GSpawnFlags ) (0),
-      NULL, NULL, &pid, NULL, &g_out, NULL, Aerror);
+      NULL, NULL, &Lpid, NULL, &g_out, NULL, Aerror);
     g_strfreev(Lcmd);
   
     if (Lbolstatus == FALSE) {
@@ -223,7 +228,7 @@ gboolean scan_for_drives(GHashTable *Ahash, GError **Aerror)
       
     g_io_channel_shutdown(Lcom, FALSE, NULL);
     g_io_channel_unref(Lcom);
-    g_spawn_close_pid(pid);
+    g_spawn_close_pid(Lpid);
     g_hash_table_remove(Ahash, "drivedesc");
   }
 
@@ -233,7 +238,7 @@ gboolean scan_for_drives(GHashTable *Ahash, GError **Aerror)
 gboolean maj_drive_info(Tdriveinfo *Adrive, gchar *Adev, GError **Aerror)
 {
   gchar *Lcommandline, *Lout = NULL, *Lerr = NULL;
-  gchar *s = NULL, *f;
+  gchar *scd = NULL, *sdvd = NULL, *f;
   gint Lexit = 0;
   gboolean Lstatus;
   gint Ltype = 0;
@@ -252,25 +257,35 @@ _DEB("==================%s\n", Lcommandline);
   if (strstr(Lout, CDRECORD_WRITE_CDR)) Ltype += _WRITE_CDR;
   if (strstr(Lout, CDRECORD_WRITE_CDRW)) Ltype += _WRITE_CDRW;
   if (strstr(Lout, CDRECORD_WRITE_DUMMY)) Ltype += _WRITE_DUMMY;
+  if (strstr(Lout, CDRECORD_READ_DVD)) Ltype += _READ_DVD;
+  if (strstr(Lout, CDRECORD_WRITE_DVD)) Ltype += _WRITE_DVD;
 
-  /* vitesse de lecture maximum */
-  if ((s=strstr(Lout, CDRECORD_MAX_CDREAD_SPEED))) {
-    s+=strlen(CDRECORD_MAX_CDREAD_SPEED);
-    if ((s=strstr(s, CDRECORD_MAX_CDREAD_SPEED_CD))) {
-      s=ltrim(s+strlen(CDRECORD_MAX_CDREAD_SPEED_CD));
+  /* vitesse de lecture maximum CD */
+  if ((scd=strstr(Lout, CDRECORD_MAX_CDREAD_SPEED))) {
+    scd+=strlen(CDRECORD_MAX_CDREAD_SPEED);
+    if ((scd=strstr(scd, CDRECORD_MAX_CDREAD_SPEED_CD))) {
+      scd=ltrim(scd+strlen(CDRECORD_MAX_CDREAD_SPEED_CD));
     }
-    f=s;
+    f=scd;
     while (isdigit(*f)) f++;
-    *f=0;
+    *(f++)=0;
+
+    if ((sdvd=strstr(f, CDRECORD_MAX_CDREAD_SPEED_DVD))) {
+      sdvd=ltrim(sdvd+strlen(CDRECORD_MAX_CDREAD_SPEED_DVD));
+    }
+    f=sdvd;
+    while (isdigit(*f)) f++;
+    *(f++)=0;
   }
   
   g_free(Lout);
   g_free(Lerr);
 
-  if (!s) return FALSE;
+  if (!scd) return FALSE;
 
   Adrive->type = Ltype; /* type: lecteur ou graveur */
-  Adrive->vitesse = atoi(s);  /* vitesse de lecture ou de gravure maximum */
+  Adrive->vitesse = atoi(scd);  /* vitesse de lecture ou de gravure maximum */
+  Adrive->vitessedvd = sdvd ? atoi(sdvd) : -1;
   _DEB("== le type => %d\n", Ltype);
   _DEB("== donc la vitesse de lecture => %d\n", Adrive->vitesse  );
   return TRUE;
@@ -290,13 +305,23 @@ gchar *burn_data_getextrapara(GHashTable *Ahash)
   return Lreturn;
 }
 
+gchar *get_blank_type(GHashTable *Ahash)
+{
+  gboolean *Luseautoblank = (gboolean *) g_hash_table_lookup(Ahash, "useautoblank");
+
+  if (!*Luseautoblank) return g_strdup("");
+
+  return g_strdup_printf("blank=%s", conf_get_boolean("fastblank") == TRUE ? "fast" : "all");
+}
+
 /* copie d'une image iso */
 gboolean burn_from_image(gchar *Aop, GHashTable *Ahash, GError **Aerror) {
   gchar **Lcmd;
   gchar *Lcommandline, *Lbufgrav, *Ltxt;
   GIOChannel *Lcom, *Lcomerr;
   guint Lcomevent, Lcomerrevent;
-  gboolean Lcont = TRUE;
+  gint *Lcont = (gint *) g_hash_table_lookup(Ahash, "cont");
+  gboolean *Labort = (gboolean *) g_hash_table_lookup(Ahash, "gabort");
   GtkWindow *Lwindow = GTK_WINDOW(g_hash_table_lookup(Ahash, "window_burn"));
   gboolean Lsimul = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sc_hash_table_lookupv(Ahash, "%ssimul", Aop)));
   gint Lnbrgravure = gtk_spin_button_get_value(GTK_SPIN_BUTTON(sc_hash_table_lookupv(Ahash, "nbrcd%s", Aop)));  
@@ -310,17 +335,20 @@ gboolean burn_from_image(gchar *Aop, GHashTable *Ahash, GError **Aerror) {
   GtkLabel *Ltitle = GTK_LABEL(g_hash_table_lookup(Ahash, "gravetitle"));
   GtkToggleButton *Lbtnnotfix = GTK_TOGGLE_BUTTON(sc_hash_table_lookupv(Ahash, "%snotfix", Aop));
   gboolean Lnotfix = Lbtnnotfix ? gtk_toggle_button_get_active(Lbtnnotfix) : FALSE;
+  gchar **Lextrapara = g_hash_table_lookup(Ahash, "extrapara");
   gboolean Lmulti = FALSE;
   gint *Ldone = (gint *)g_hash_table_lookup(Ahash, "done"); /* fais */
   gint Lcurcd;
   gint Lnbrpass=1;
-  gchar *Lbufvitesse;
+  gchar *Lbufvitesse, *Lblank;
   gchar Llasttrack[_BUF_SIZE] = "00";
   gint g_out, g_err, Lnbrarg;
   gboolean Lstatus = FALSE;
   gboolean Leject = FALSE;
   GtkToggleButton *Ltbtn = NULL;
 
+  Lblank = get_blank_type(Ahash);
+
   if ((Ltbtn = GTK_TOGGLE_BUTTON(sc_hash_table_lookupv(Ahash, "%smulti", Aop))))
     Lmulti =gtk_toggle_button_get_active(Ltbtn);
 
@@ -329,6 +357,7 @@ gboolean burn_from_image(gchar *Aop, GHashTable *Ahash, GError **Aerror) {
   for (Lcurcd=1; Lcurcd<= Lnbrgravure;
       ((Lsimul && Lnbrpass>1) || (!Lsimul)) ? ( Lcurcd++, Lnbrpass=1 ) : ( Lnbrpass++ )) {
 
+    Ldosimul = (Lsimul && Lnbrpass==1);
     if (Lcurcd > 1 && !Ldosimul) {
       /* copie sur un nouveau cd, on demande a l'utilisateur d'inserer le
        * nouveau cd vierge */
@@ -340,14 +369,13 @@ gboolean burn_from_image(gchar *Aop, GHashTable *Ahash, GError **Aerror) {
       Lrep = gtk_dialog_run(GTK_DIALOG(Lconfirm));
       gtk_widget_destroy(Lconfirm);
       if (Lrep == GTK_RESPONSE_NO) {
-        gboolean *Labort = (gboolean *) g_hash_table_lookup(Ahash, "gabort");
         /* si c'est non alors on arrete */
         *Labort = TRUE;
-        return TRUE;
+        Lstatus = TRUE;
+        break;
       }
 
     }
-    Ldosimul = (Lsimul && Lnbrpass==1);
 
     /* faut il ejecter le CD-R apres l'operation ?
      * oui si l'utilisateur a cocher la case "ejecter le cd" ou
@@ -363,13 +391,16 @@ gboolean burn_from_image(gchar *Aop, GHashTable *Ahash, GError **Aerror) {
 
     Lbufvitesse = get_combo_value(Lvitesse);
     Lbufgrav = get_combo_value(Lgraveur);
-    Lcommandline = g_strdup_printf("%s dev=%s -v gracetime=2 %s%s %s %s %s %s %s '%s'",
+    Lcommandline = g_strdup_printf("%s dev=%s -v gracetime=2 %s%s %s %s %s %s %s %s %s %s '%s'",
         conf_get_string("cdrecord"), Lbufgrav, *Lbufvitesse != '0' ? "speed=" : "", 
         *Lbufvitesse != '0' ? Lbufvitesse : "",  
         Ldosimul ? " -dummy" : "",  /* simulation ? */
         Ldosimul || Lnotfix ? " -nofix" : "", /* fixer le cd apres ecriture ? */
         Lmulti ? " -multi" : "",  /* multi-session */
         Leject ? "-eject" : "", /* ejecter le cd apres l'operation */
+        Lblank,
+        Lextrapara && *Lextrapara ? *Lextrapara : "",  /* parametre supplementaire tel que le mode d'ecriture */
+        conf_get_boolean("overburn") ? "-overburn" : "",
         conf_get_string("cdrecordpara"),
         *Liso);
   _DEB("execution [%s]\n", Lcommandline);
@@ -380,21 +411,20 @@ gboolean burn_from_image(gchar *Aop, GHashTable *Ahash, GError **Aerror) {
     Lstatus = g_shell_parse_argv(Lcommandline, &Lnbrarg, &Lcmd, Aerror);
     g_free(Lcommandline);
     if (Lstatus == FALSE) {
-      return FALSE;
+      break;
     }
 
     Lstatus = g_spawn_async_with_pipes(NULL, Lcmd, NULL, /* env argument */
-        (GSpawnFlags ) (0),
+        (GSpawnFlags ) (G_SPAWN_DO_NOT_REAP_CHILD),
         NULL, NULL, Lpid, NULL, &g_out, &g_err, Aerror);
     g_strfreev(Lcmd);
 
     if (Lstatus == FALSE) {
       g_warning("ERROR EXECUTION !\n");
-      return FALSE;
+      break;
     }
 
-    Lcont = TRUE;
-    g_hash_table_insert(Ahash, "cont", &Lcont);
+    *Lcont = 1;
     g_hash_table_insert(Ahash, "lasttrack", &Llasttrack);
     Lcom = g_io_channel_unix_new( g_out );
     g_io_channel_set_encoding (Lcom, NULL, NULL);
@@ -410,9 +440,11 @@ gboolean burn_from_image(gchar *Aop, GHashTable *Ahash, GError **Aerror) {
     Lcomerrevent = g_io_add_watch (Lcomerr, (G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_PRI),
                                       cdrecord_grave_callback, Ahash); 
     
-    while (Lcont) {
+    while (*Lcont > 0 && *Labort == FALSE) {
       gtk_main_iteration();    
     }
+    exit_prog(*Lpid, Aerror, NULL);
+
     g_source_remove(Lcomevent);
     g_source_remove(Lcomerrevent);
       
@@ -424,11 +456,11 @@ gboolean burn_from_image(gchar *Aop, GHashTable *Ahash, GError **Aerror) {
     *Lpid = 0;
 
     g_hash_table_remove(Ahash, "lasttrack");
-    g_hash_table_remove(Ahash, "cont");
 
     if (*Aerror) {
       _DEB("IL Y A UNE ERREUR !!");
-      return FALSE;
+      Lstatus = FALSE;
+      break; 
     }
 
     if (*Loperationstatus == FALSE) {
@@ -436,7 +468,8 @@ gboolean burn_from_image(gchar *Aop, GHashTable *Ahash, GError **Aerror) {
        * dans tous les cas ce n'est pas normal, on genere une erreur */
       g_set_error(Aerror, GRAVEMAN_ERROR, _ERR_UNKNOWN_ERROR, 
         _("Communication error with cdrecord. Check that you have access to cdrecord release 2.0 !"));
-      return FALSE;
+      Lstatus = FALSE;
+      break;
     }
 
     if (Ldosimul) {
@@ -450,10 +483,10 @@ gboolean burn_from_image(gchar *Aop, GHashTable *Ahash, GError **Aerror) {
       Lrep = gtk_dialog_run(GTK_DIALOG(Lconfirm));
       gtk_widget_destroy(Lconfirm);
       if (Lrep == GTK_RESPONSE_NO) {
-        gboolean *Labort = (gboolean *) g_hash_table_lookup(Ahash, "gabort");
         /* si c'est non alors on arrete */
         *Labort = TRUE;
-        return TRUE;
+        Lstatus = TRUE;
+        break;
       }
     }
 
@@ -461,6 +494,8 @@ gboolean burn_from_image(gchar *Aop, GHashTable *Ahash, GError **Aerror) {
     strcpy(Llasttrack, "00");
   }
 
+  g_free(Lblank);
+  
   return Lstatus;
 }
 
@@ -485,7 +520,7 @@ gboolean cdrecord_grave_callback(GIOChannel *Astd, GIOCondition Acond, gpointer
   GIOStatus Lstatus;
   GHashTable *Lhash = (GHashTable *)Adata;
   gchar Lbuffer[_BUF_SIZE];
-  gboolean *Lcont = (gboolean *) g_hash_table_lookup(Lhash, "cont"); /* on traite encore des donnees ? */
+  gint *Lcont = (gint *) g_hash_table_lookup(Lhash, "cont"); /* on traite encore des donnees ? */
   GError **Lerreur = (GError **) g_hash_table_lookup(Lhash, "gerror"); /* pointeur erreur */
   gint *Ltodo = (gint *) g_hash_table_lookup(Lhash, "todo"); /* nombre de piste a traiter */
   gint *Ldone = (gint *) g_hash_table_lookup(Lhash, "done"); /* nombre de piste deja traite */
@@ -505,7 +540,7 @@ gboolean cdrecord_grave_callback(GIOChannel *Astd, GIOCondition Acond, gpointer
   
   /* fin du callback lorsque l'on recoi un signal comme quoi le pipe est ferme */
   if (Acond == G_IO_HUP || Acond == G_IO_ERR) {
-    (*Lcont) = FALSE;
+    (*Lcont) = 0;
     return FALSE;
   }
   
@@ -606,43 +641,51 @@ _DEB("===>%s", Lbuffer);
   } else if (strstr(Lbuffer, CDRECORD_FIXATING)) {
     /* cloture du cd */
     gtk_label_set_text(Ltitle, _("Fixating..."));
+  } else if (strstr(Lbuffer, CDRECORD_BLANKING)) {
+    /* blanking disk */
+    gtk_label_set_text(Ltitle, _("Blanking..."));
   } else if ((f=strstr(Lbuffer, CDRECORD_STATUS))) {
     gboolean *Lstatus = (gboolean *) g_hash_table_lookup(Lhash, "operationstatus");
-
+/* bug 11803
+ * cause des problems apparements, a tester .. 
     f=f+strlen(CDRECORD_STATUS);
-    /* fin de la gravure */
+     fin de la gravure 
     if (*f != '0') {
       g_set_error(Lerreur, GRAVEMAN_ERROR, _ERR_BUF_EMPTY, _("Error writing tracks, buffer was empty !"));
       return FALSE;
-    }
+    } */
     /* ok tout s'est bien passe ! */
     *Lstatus = TRUE;
   } else if (strstr(Lbuffer, CDRECORD_INPUTOUTPUT_ERROR)) {
     /* erreur entre sortie */
     g_set_error(Lerreur, GRAVEMAN_ERROR, _ERR_NO_CD, _("Error writing CD !"));
-    (*Lcont) = FALSE;
+    (*Lcont) = 0;
     return FALSE;
-  } else if (strstr(Lbuffer, CDRECORD_ERRORDISK)) {
+  } else if (strstr(Lbuffer, CDRECORD_CANNOT_BLANK)) {
+    g_set_error(Lerreur, GRAVEMAN_ERROR, _ERR_CANNOT_BLANK, _("Cannot blank disk, aborting."));
+    (*Lcont) = 0;
+    return FALSE;
+  } else if ((strstr(Lbuffer, CDRECORD_ERRORDISK)) || (strstr(Lbuffer, CDRECORD_INCOMPATIBLE_MEDIUM))) {
     /* erreur pas de cd vierge */
     g_set_error(Lerreur, GRAVEMAN_ERROR, _ERR_NO_CD, _("Error, a CD-R/CD-RW is required in the cd recorder !"));
-    (*Lcont) = FALSE;
+    (*Lcont) = 0;
     return FALSE;
   } else if (strstr(Lbuffer, CDRECORD_BAD_RECORDER)) {
     /* erreur pas de cd vierge */
     g_set_error(Lerreur, GRAVEMAN_ERROR, _ERR_BAD_RECORDER, _("CD recorder unsupported !"));
-    (*Lcont) = FALSE;
+    (*Lcont) = 0;
     return FALSE;
   } else if ((strstr(Lbuffer, CDRECORD_INAPPROPRIATE))) {
     /* erreur fichier audio non compatible */
     g_snprintf(Lsbuf, sizeof(Lsbuf)-1, _("Error with track %.0f: inappropriate audio coding !"), Ltotaldone);
     g_set_error(Lerreur, GRAVEMAN_ERROR, _ERR_INAPPROPRIATE_DATA, Lsbuf);
-    (*Lcont) = FALSE;
+    (*Lcont) = 0;
     return FALSE;
   } else if ((f=strstr(Lbuffer, CDRECORD_NOSUCHFILE))) { 
     /* erreur fichier image iso source introuvable ! */
     g_snprintf(Lsbuf, sizeof(Lsbuf)-1, _("Cannot open iso image !"));
     g_set_error(Lerreur, GRAVEMAN_ERROR, _ERR_INAPPROPRIATE_DATA, Lsbuf);
-    (*Lcont) = FALSE;
+    (*Lcont) = 0;
     return FALSE;
   } else if ((f=strstr(Lbuffer, conf_get_string("cdrecord")))) {
     f=f+strlen(conf_get_string("cdrecord"))+1;
@@ -651,7 +694,7 @@ _DEB("===>%s", Lbuffer);
       Ltxt = _UTF8(f);
       g_set_error(Lerreur, GRAVEMAN_ERROR, _ERR_INAPPROPRIATE_DATA, Ltxt);
       g_free(Ltxt);
-      (*Lcont) = FALSE;
+      (*Lcont) = 0;
       return FALSE;
     }
   } 
@@ -666,12 +709,13 @@ gboolean gravepiste(GHashTable *Ahash, GError **Aerror) {
   GtkWidget *Lgraveur = g_hash_table_lookup(Ahash, "dstaudiocombo");
   gchar **Lrepertoire = (gchar **)g_hash_table_lookup(Ahash, "tmpdir");
   gboolean *Loperationstatus = (gboolean *)g_hash_table_lookup(Ahash, "operationstatus");
+  gboolean *Labort = (gboolean *) g_hash_table_lookup(Ahash, "gabort");
   gchar *Lbuftitle = NULL;
   gchar **Lcmd;
   gchar *Lcommandline;
   GIOChannel *Lcom, *Lcomerr;
   guint Lcomevent, Lcomerrevent;
-  gboolean Lcont;
+  gint *Lcont = (gint *) g_hash_table_lookup(Ahash, "cont"); /* on traite encore des donnees ? */
   /* nombre d'element total */
   gint *Ldone = (gint *)g_hash_table_lookup(Ahash, "done"); /* fais */
   gint *Ltodo = (gint *) g_hash_table_lookup(Ahash, "todo");
@@ -682,18 +726,20 @@ gboolean gravepiste(GHashTable *Ahash, GError **Aerror) {
   gchar *Lbufvitesse, *Lbufgrav;
   gboolean Lsimul = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(g_hash_table_lookup(Ahash, "audiosimul")));
   gboolean Lnotfix = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(g_hash_table_lookup(Ahash, "audionotfix")));
+  gboolean Ldaomode = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(g_hash_table_lookup(Ahash, "audiodao")));
   gint Lnbrgravure = gtk_spin_button_get_value(GTK_SPIN_BUTTON(g_hash_table_lookup(Ahash, "nbrcdaudio")));
   gboolean Lwanteject = conf_get_boolean("eject");
   gint Lcurcd;
   gboolean Ldosimul;
   gint Lnbrpass=1;
-  gchar *Lfileslist;
+  gchar *Lfileslist, *Lblank;
   gboolean Leject = FALSE;
   gchar Llasttrack[_BUF_SIZE] = "00";
 
   Lfileslist = make_audio_fileslist(*Ltodo);
   Lbufvitesse = get_combo_value(Lvitesse);
   Lbufgrav = get_combo_value(Lgraveur);
+  Lblank = get_blank_type(Ahash);
 
   *Ltodo = (*Ltodo) * ((Lnbrgravure) + (Lsimul ? 1 : 0));
 
@@ -701,8 +747,26 @@ gboolean gravepiste(GHashTable *Ahash, GError **Aerror) {
   for (Lcurcd=1; Lcurcd<= Lnbrgravure;
       ((Lsimul && Lnbrpass>1) || (!Lsimul)) ? ( Lcurcd++, Lnbrpass=1 ) : ( Lnbrpass++ )) { 
     *Loperationstatus = FALSE;
-
     Ldosimul = (Lsimul && Lnbrpass==1);
+
+    if (Lcurcd > 1 && !Ldosimul) {
+      /* copie sur un nouveau cd, on demande a l'utilisateur d'inserer le
+       * nouveau cd vierge */
+      gint Lrep;
+      GtkWidget *Lconfirm = gtk_message_dialog_new(Lwindow,
+                                              GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION,
+                                              GTK_BUTTONS_YES_NO,
+                 _("Now you should insert next CD-R, do you want to continue operation ?"));
+      Lrep = gtk_dialog_run(GTK_DIALOG(Lconfirm));
+      gtk_widget_destroy(Lconfirm);
+      if (Lrep == GTK_RESPONSE_NO) {
+        /* si c'est non alors on arrete */
+        *Labort = TRUE;
+        Lstatus = TRUE;
+      }
+
+    }
+
     Lbuftitle = get_title(Lnbrgravure, Lcurcd, Ldosimul);
     gtk_label_set_text(GTK_LABEL(Ltitle), Lbuftitle);
     g_free(Lbuftitle);
@@ -712,32 +776,34 @@ gboolean gravepiste(GHashTable *Ahash, GError **Aerror) {
      * si il faut realiser d'autre gravure sur d'autres CD-R */
     Leject = ((Lcurcd > 1 && (Lwanteject || Lcurcd<Lnbrgravure)) || (Lcurcd == 1 && !Ldosimul && Lwanteject));
 
-    Lcommandline = g_strdup_printf("%s dev=%s -v gracetime=2 %s%s %s%s %s %s -pad -audio %s", 
+    Lcommandline = g_strdup_printf("%s dev=%s -v gracetime=2 %s%s %s%s %s %s %s %s %s -pad -audio %s", 
         conf_get_string("cdrecord"), Lbufgrav, *Lbufvitesse == '0' ? "" : "speed=", *Lbufvitesse == '0' ? "" : Lbufvitesse,
         Ldosimul ? " -dummy" : "",  /* simulation ? */ 
         Ldosimul || Lnotfix ? " -nofix" : "", /* fixer le cd apres ecriture ? */
+        Ldaomode ? " -dao" : "",  /* ecriture en mode DAO, pas de pause entre les pistes */
         Leject ? "-eject" : "", /* ejecter le cd apres l'operation */
+        Lblank, /* effacer le cdrw avant ? */
         conf_get_string("cdrecordpara"),
+        conf_get_boolean("overburn") ? "-overburn" : "",
         Lfileslist);
 _DEB("execution [%s]\n", Lcommandline);
     Lstatus = g_shell_parse_argv(Lcommandline, &Lnbrarg, &Lcmd, Aerror);
     g_free(Lcommandline);
     if (Lstatus == FALSE) {
-      return FALSE;
+      break;
     }
 
     Lstatus = g_spawn_async_with_pipes(*Lrepertoire, Lcmd, NULL, /* env argument */
-        (GSpawnFlags ) (0),
+        (GSpawnFlags ) (G_SPAWN_DO_NOT_REAP_CHILD),
         NULL, NULL, Lpid, NULL, &g_out, &g_err, Aerror);
     g_strfreev(Lcmd);
 
     if (Lstatus == FALSE) {
       g_warning("ERROR EXECUTION !\n");
-      return FALSE;
+      break;
     }
 
-    Lcont = TRUE;
-    g_hash_table_insert(Ahash, "cont", &Lcont);
+    *Lcont = 1;
     g_hash_table_insert(Ahash, "lasttrack", &Llasttrack);
     Lcom = g_io_channel_unix_new( g_out );
     g_io_channel_set_encoding (Lcom, NULL, NULL);
@@ -753,9 +819,10 @@ _DEB("execution [%s]\n", Lcommandline);
     Lcomerrevent = g_io_add_watch (Lcomerr, (G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_PRI),
                                         cdrecord_grave_callback, Ahash);  
     
-    while (Lcont) {
+    while (*Lcont>0 && *Labort == FALSE) {
       gtk_main_iteration(); 
     }
+    exit_prog(*Lpid, Aerror, NULL);
 
     g_source_remove(Lcomerrevent);
     g_source_remove(Lcomevent);
@@ -768,7 +835,6 @@ _DEB("execution [%s]\n", Lcommandline);
     *Lpid = 0;
 
     g_hash_table_remove(Ahash, "lasttrack");
-    g_hash_table_remove(Ahash, "cont");
 
     if (*Aerror) {
       Lstatus = FALSE;
@@ -794,7 +860,6 @@ _DEB("execution [%s]\n", Lcommandline);
       Lrep = gtk_dialog_run(GTK_DIALOG(Lconfirm));
       gtk_widget_destroy(Lconfirm);
       if (Lrep == GTK_RESPONSE_NO) {
-        gboolean *Labort = (gboolean *) g_hash_table_lookup(Ahash, "gabort");
         /* si c'est non alors on arrete */
         *Labort = TRUE;
         Lstatus = TRUE;
@@ -805,6 +870,7 @@ _DEB("execution [%s]\n", Lcommandline);
     strcpy(Llasttrack, "00");
   }
 
+  g_free(Lblank);
   g_free(Lbufgrav);
   g_free(Lbufvitesse);
   g_free(Lfileslist);
@@ -869,15 +935,17 @@ gboolean perform_erase_cdrw(GHashTable *Ahash, GError **Aerror)
   gint g_out, g_err, Lnbrarg;
   gint *Lcont = g_hash_table_lookup(Ahash, "cont");
   GtkWidget *Lgraveur = g_hash_table_lookup(Ahash, "dstothercombo");
+  gboolean *Lfastblank = g_hash_table_lookup(Ahash, "usefastblank");
 
   /* pid de cdrecord */
   gint *Lpid = (gint *) g_hash_table_lookup(Ahash, "pid");
 
   Lbufgrav = get_combo_value(Lgraveur);
   Lcommandline = g_strdup_printf("%s dev=%s -v gracetime=2 blank=%s",
-      conf_get_string("cdrecord"), Lbufgrav, conf_get_boolean("fastblank") ? "fast" : "all");
+      conf_get_string("cdrecord"), Lbufgrav, (*Lfastblank == TRUE ? "fast" : "all"));
   g_free(Lbufgrav);
 
+  printf("execution [%s]\n", Lcommandline);
   _DEB("execution [%s]\n", Lcommandline);
   Lstatus = g_shell_parse_argv(Lcommandline, &Lnbrarg, &Lcmd, Aerror);
   g_free(Lcommandline);
@@ -887,7 +955,7 @@ gboolean perform_erase_cdrw(GHashTable *Ahash, GError **Aerror)
   }
 
   Lstatus = g_spawn_async_with_pipes(NULL, Lcmd, NULL, /* env argument */
-      (GSpawnFlags ) (0),
+      (GSpawnFlags ) (G_SPAWN_DO_NOT_REAP_CHILD),
        NULL, NULL, Lpid, NULL, &g_out, &g_err, Aerror);
   g_strfreev(Lcmd);
 
@@ -913,6 +981,8 @@ _DEB("cont vaut => %d\n", *Lcont);
 _DEB("fin !!!\n");
   while (*Lcont > 0) { while(gtk_events_pending()) gtk_main_iteration(); }
 
+  exit_prog(*Lpid, Aerror, NULL);
+
   g_source_remove(Lcomerrevent);
   g_source_remove(Lcomevent);
 
@@ -954,7 +1024,7 @@ gboolean perform_fix_cd(GHashTable *Ahash, GError **Aerror)
   }
 
   Lstatus = g_spawn_async_with_pipes(NULL, Lcmd, NULL, /* env argument */
-      (GSpawnFlags ) (0),
+      (GSpawnFlags ) (G_SPAWN_DO_NOT_REAP_CHILD),
        NULL, NULL, Lpid, NULL, &g_out, &g_err, Aerror);
   g_strfreev(Lcmd);
 
@@ -980,6 +1050,8 @@ _DEB("cont vaut => %d\n", *Lcont);
 _DEB("fin !!!\n");
   while (*Lcont > 0) { while(gtk_events_pending()) gtk_main_iteration(); }
 
+  exit_prog(*Lpid, Aerror, NULL);
+
   g_source_remove(Lcomerrevent);
   g_source_remove(Lcomevent);
 
@@ -994,6 +1066,16 @@ _DEB("fin !!!\n");
 }
 
 
+/* ejecter un CD */
+void eject_cd(gchar *Adev, GError **Aerror)
+{
+  gchar *Lcommandline = g_strdup_printf("%s -eject dev=%s", conf_get_string("cdrecord"), Adev);
+  _DEB("EJECT %s\n", Adev);
+  
+  g_spawn_command_line_sync(Lcommandline, NULL, NULL, NULL, Aerror);
+
+  g_free(Lcommandline);
+}
 
 /*
  * vim:et:ts=8:sts=2:sw=2
